<?php

/**
 * Template
 *
 * Generates markup for display based on a set of input values
 *
 * Template subclasses are intended to work as a kind of fixed contract between
 * frontend and backend code. This is intended to help minimise the occurrence
 * of easily-missed bugs caused by E_NOTICE errors hidden away in attributes of
 * HTML tags.
 *
 * @abstract
 * @author     Henry Smith <henry@henrysmith.org> 
 * @copyright  2011 Henry Smith
 * @license    GPLv2.0
 * @package    Tree
 * @subpackage Component
 * @version    0.02
 */
namespace Tree\Component;

use \stdClass;
use \ArrayAccess;
use \Tree\Behaviour\PreTemplateDisplayLogic;
use \Tree\Exception\TemplateException;

abstract class Template implements ArrayAccess {

	/**
	 * An associative array mapping names of optional input values to
	 * defaults for those values
	 * 
	 * @access protected
	 * @var    array
	 */
	protected $optionalInputValues = array();

	/**
	 * An array listing required input values without which the template
	 * must not attempt to generate markup
	 * 
	 * @access protected
	 * @var    array
	 */
	protected $requiredInputValues = array();

	/**
	 * The name of the template file from which output is to be generated 
	 * 
	 * @access protected
	 * @var    string
	 */
	protected $templateFilename;

	/**
	 * An associative array of actual input values 
	 * 
	 * @access private
	 * @var    array
	 */
	private $inputValues = array();

	/**
	 * The string generated by the last invocation of generateMarkup()
	 * 
	 * @access private
	 * @var    string
	 */
	private $outputString;

	/**
	 * An array listing the stylesheets on which the template HTML depends in
	 * order to display correctly
	 * 
	 * @access private
	 * @var    array
	 */
	private $stylesheetDependencies = array();

	/**
	 * An array listing the Javascript files on which the template HTML depends
	 * 
	 * @access private
	 * @var    array
	 */
	private $javascriptDependencies = array();

	/**
	 * An associative array of input values that are common to all templates
	 *
	 * @access private
	 * @var    array
	 */
	private static $globalValues = array();

	/**
	 * The absolute path of the directory containing the template files
	 *
	 * This path should be such that $templateDirectory . DIRECTORY_SEPARATOR .
	 * $templateFilename constitutes a correct absolute path to the template
	 *
	 * If this isn't set, Template will still attempt to find the template file
	 * by scanning the include_path, which may work if PHP is correctly configured
	 * but isn't ideal from a performance point-of-view.
	 *
	 * @access private
	 * @var    array
	 * @static
	 */
	private static $templateDirectory;

	public function __construct()
	{
		$this->inputValues = $this->optionalInputValues;
	}

	/**
	 * Returns the input value of the given name
	 * 
	 * @access public
	 * @param  string $name 
	 * @return string
	 */
	public function getInputValue($name)
	{
		if (isset($this->inputValues[$name])) {
			return $this->inputValues[$name];
		} else {
			return null;
		}
	}

	/**
	 * Returns the output of the template if all required input values
	 * are available
	 * 
	 * @access public
	 * @param  boolean $regenerate [optional]  Whether the output generated by previous invocation should be disregarded
	 * @return string
	 */
	public function getOutput($regenerate = false)
	{
		if ($regenerate === false && $this->outputString !== null) {
			return $this->outputString;
		}

		$this->outputString = $this->generateOutput();

		return $this->outputString;
	}

	/**
	 * Returns an array of the Javascript files on which the template depends
	 * 
	 * @access public
	 * @return array
	 */
	public function getJavascriptDependencies()
	{
		return $this->javascriptDependencies;
	}

	/**
	 * Returns an array of the stylesheets on which the template depends
	 * 
	 * @access public
	 * @return array
	 */
	public function getStylesheetDependencies()
	{
		return $this->stylesheetDependencies;
	}

	/**
	 * Adds a Javascript file to the list of those on which the template depends
	 * 
	 * @access public
	 * @param  string $filename 
	 */
	public function requireJavascript($filename)
	{
		$this->javascriptDependencies[] = $filename;
	}

	/**
	 * Adds a stylesheet to the list of those on which the template depends
	 * 
	 * @access public
	 * @param  string $filename             The name of the CSS file
	 * @param  string $media    [optional]  The CSS media type
	 */
	public function requireStylesheet($filename, $media = 'all')
	{
		$stylesheet        = new stdClass;
		$stylesheet->href  = $filename;
		$stylesheet->media = $media;

		$this->stylesheetDependencies[] = $stylesheet;
	}

	/**
	 * Stores an input value to be used by the template 
	 *
	 * Throws an exception if the name of the value given is not a valid
	 * one for this template.
	 * 
	 * @access public
	 * @param  string $name 
	 * @param  mixed  $value 
	 */
	public function setInputValue($name, $value)
	{
		if (!$this->isPermissibleValue($name)) {
			$message = "Invalid template value: {$name}";
			$code    = TemplateException::INVALID_VALUE_NAME;
			throw new TemplateException($message, $code, $this);
		}

		$this->inputValues[$name] = $value;
	}

	/**
	 * Removes an input value from the template
	 * 
	 * @access public
	 * @param  string $name  The name of the value to unset
	 */
	public function unsetInputValue($name)
	{
		if (!$this->isPermissibleValue($name)) {
			$message = "Invalid template value: {$name}";
			$code    = TemplateException::INVALID_VALUE_NAME;
			throw new TemplateException($message, $code, $this);
		}

		if (isset($this->inputValues[$name])) {
			unset($this->inputValues[$name]);
		}
	}


	/**
	 * Sets the name of the template file to use when generating output 
	 *
	 * The filename given should either be a file in PHP's include_path or
	 * an absolute filename.
	 * 
	 * @access public
	 * @param  string $templateFilename 
	 */
	public function setTemplateFilename($templateFilename)
	{
		$this->templateFilename = $templateFilename;
	}

	/**
	 * Stores an input value to be used by all template subclasses
	 * 
	 * @access public
	 * @param  string $name 
	 * @param  mixed $value 
	 * @static
	 */
	public static function setGlobalValue($name, $value)
	{
		self::$globalValues[$name] = $value;
	}

	public static function getGlobalValue($name)
	{
		if (isset(self::$globalValues[$name])) {
			return self::$globalValues[$name];
		} else {
			return null;
		}
	}

	/**
	 * Stores the given directory as the one to look in to find the template
	 * filename as specified by $this->templateFilename
	 * 
	 * @static
	 * @access public
	 * @param  string $directory 
	 */
	public static function setTemplateDirectory($directory)
	{
		self::$templateDirectory = $directory;
	}

	/**
	 * ArrayAccess: Indicates whether the given template input value has been set
	 * 
	 * @access public
	 * @param  string $offset 
	 * @return boolean
	 */
	public function offsetExists($offset)
	{
		return isset($this->inputValues[$offset]);
	}

	/**
	 * ArrayAccess: Returns the template input value of the given name
	 * 
	 * @access public
	 * @param  string $offset 
	 * @return mixed
	 */
	public function offsetGet($offset)
	{
		return $this->getInputValue($offset);
	}

	/**
	 * ArrayAccess: Stores the given template input value
	 * 
	 * @access public
	 * @param  string $offset  The name of the value to store
	 * @param  mixed  $value   The value itself
	 */
	public function offsetSet($offset, $value)
	{
		$this->setInputValue($offset, $value);
	}

	/**
	 * ArrayAccess: Removes the given template input value
	 * 
	 * @access public
	 * @param  string $offset 
	 */
	public function offsetUnset($offset)
	{
		$this->unsetInputValue($offset);
	}

	/**
	 * Finds the absolute path of the template from the relative filename in
	 * $this->templateFilename
	 * 
	 * @access private
	 * @return string
	 */
	private function findAbsolutePath()
	{
		if (isset(self::$templateDirectory)) {

			$templateDirectory = self::$templateDirectory;
			$templateDirectory = rtrim($templateDirectory, DIRECTORY_SEPARATOR);

			$absolutePath = $templateDirectory
				. DIRECTORY_SEPARATOR
				. $this->templateFilename;

			return $absolutePath;
		} else {
			$message = 'No template directory set';
			$code    = TemplateException::TEMPLATE_DIRECTORY_NOT_SET;
			throw new TemplateException($message, $code, $this);
		}
	}

	/**
	 * Generates the output string provided that all the required input
	 * values are available
	 *
	 * Throws an exception if required input is missing or if no template
	 * filename has been set.
	 * 
	 * @access private
	 * @return string
	 */
	private function generateOutput()
	{
		if ($this instanceof PreTemplateDisplayLogic) {
			$this->preProcess();
		}

		if ($this->templateFilename === null) {
			$message = 'No template file specified';
			$code    = TemplateException::MISSING_TEMPLATE_FILENAME;
			throw new TemplateException($message, $code, $this);
		}

		$absolutePath = $this->findAbsolutePath();

		if (!file_exists($absolutePath)) {
			$message = 'Template file not found';
			$code    = TemplateException::TEMPLATE_NOT_FOUND;
			throw new TemplateException($message, $code, $this);
		}

		if (!is_readable($absolutePath)) {
			$message = "Template not readable";
			$code    = TemplateException::TEMPLATE_NOT_READABLE;
			throw new TemplateException($message, $code, $this);
		}

		$missingValues = $this->getMissingValues();

		if (!empty($missingValues)) {
			$message = "Missing template values: {$missingValues}";
			$code    = TemplateException::MISSING_REQUIRED_VARIABLE;
			throw new TemplateException($message, $code, $this);
		}

		extract(self::$globalValues);
		extract($this->inputValues);

		try {

			ob_start();
			include $absolutePath;

		} catch (Exception $e) {

			// inspired by Kohana_View's method of cleanly handling
			// errors that occur during output phases
			ob_end_clean();
			throw $e;

		}

		$output = ob_get_clean();

		return $output;

	}

	/**
	 * Returns an array listing the names of any required input values that
	 * have not yet been set
	 * 
	 * @access private
	 * @return array
	 */
	private function getMissingValues()
	{
		$valuesProvided = array_keys($this->inputValues);
		$valuesRequired = $this->requiredInputValues;
		$valuesMissing  = array_diff($valuesRequired, $valuesProvided);

		return $valuesMissing;
	}

	/**
	 * Indicates whether the given input value name is a valid one for this
	 * template
	 *
	 * To be valid it must either be the name of a required value or an
	 * optional value.
	 * 
	 * @access private
	 * @param  string $name 
	 * @return boolean
	 */
	private function isPermissibleValue($name)
	{
		$optionalNames = array_keys($this->optionalInputValues);
		$requiredNames = $this->requiredInputValues;

		$permissibleNames = array_merge(
			$optionalNames,
			$requiredNames
		);

		return in_array($name, $permissibleNames);
	}

}

