<?php

namespace Tree\Test\DatabaseQuery;

require_once '../Database/Connection.php';
require_once '../Database/Query.php';
require_once '../Database/Query/Select.php';
require_once '../Database/Query/Predicate.php';
require_once '../Database/Query/Join.php';
require_once 'Fake/Connection.php';

use \Tree\Database\Query;
use \Tree\Database\Query_Select;
use \Tree\Database\Query_Join;
use \PHPUnit_Framework_TestCase;
use \Tree\Test\Fake_Connection;

/**
 * QuerySelectTest
 *
 * Verifies that the SQL generated by Query_Select is as expected
 * 
 * @author     Henry Smith <henry@henrysmith.org> 
 * @copyright  2011 Henry Smith
 * @license    GPLv2.0
 * @package    Tree
 * @subpackage Test
 * @uses       \PHPUnit_Framework_TestCase
 * @version    0.00
 */
class QuerySelectTest extends PHPUnit_Framework_TestCase {

	public function setUp()
	{
		$this->db = new Fake_Connection;
	}

	/**
	 * Verifies that the most basic SELECT query possible is generated properly
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getColumnExpressions
	 * @covers \Tree\Database\Query_Select::getFromClause
	 */
	public function testBasicSelectQuery()
	{
		$query = new Query_Select($this->db);

		$query->select('*')->from('sometable');

		$sql = $query->getSql();

		$this->assertEquals("SELECT *\nFROM `sometable`\n", $sql);
	}

	/**
	 * Verifies that table name aliasing works
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getFromClause
	 */
	public function testTableNameAlias()
	{
		$query = new Query_Select($this->db);

		$query->select('*')->from(array('sometable' => 'alias'));

		$sql = $query->getSql();

		$this->assertEquals("SELECT *\nFROM `sometable` `alias`\n", $sql);
	}

	/**
	 * Verifies that ORDER BY expressions are generated properly
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::orderBy
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getOrderExpressions
	 */
	public function testOrderBy()
	{
		$query = new Query_Select($this->db);

		$query->select('*');
		$query->from('sometable');
		$query->orderBy('test', 'asc');

		$sql = $query->getSql();

		$this->assertEquals("SELECT *\nFROM `sometable`\nORDER BY `test` ASC\n", $sql);
	}

	/**
	 * Verifies that WHERE expressions are generated properly
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::where
	 * @covers \Tree\Database\Query_Select::orWhere
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getWhereExpression
	 */
	public function testWhereExpression()
	{
		$query = new Query_Select($this->db);

		$query->select('*');
		$query->from('sometable');
		$query->where('id > %d', 10);
		$query->where('title = %s', 'example');
		$query->orWhere('status = %d', 2);

		$expected  = "SELECT *\n";
		$expected .= "FROM `sometable`\n";
		$expected .= "WHERE id > 10\n";
		$expected .= "AND title = 'example'\n";
		$expected .= "OR status = 2\n";

		$actual = $query->getSql();

		$this->assertEquals($expected, $actual);
	}

	/**
	 * Verifies that LIMIT expressions are generated properly
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::limit
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getLimitExpression
	 */
	public function testLimitExpression()
	{
		$query = new Query_Select($this->db);
		$query->select('*');
		$query->from('sometable');
		$query->limit(10);

		$expected  = "SELECT *\n";
		$expected .= "FROM `sometable`\n";
		$expected .= "LIMIT 10\n";

		$actual = $query->getSql();

		$this->assertEquals($expected, $actual);
	}

	/**
	 * Verifies that SELECT interacts well enough with Query_Join to be able
	 * to add a simple join
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::addJoin
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getJoinExpressions
	 */
	public function testSingleSimpleJoin()
	{
		$query = new Query_Select($this->db);
		$query->select('*');
		$query->from(array('sometable' => 'st'));

		$join = new Query_Join($this->db);
		$join->setTable('othertable', 'ot');
		$join->setType('LEFT');
		$join->on('st.ot_id = ot.ot_id');

		$query->addJoin($join);

		$expected  = "SELECT *\n";
		$expected .= "FROM `sometable` `st`\n";
		$expected .= "LEFT JOIN `othertable` `ot`\n";
		$expected .= "ON st.ot_id = ot.ot_id\n";

		$actual = $query->getSql();

		$this->assertEquals($expected, $actual);
	}

	/**
	 * Verifies that SELECT interacts well enough with Query_Join to be able
	 * to add a longer, more involved join expression
	 *
	 * @covers \Tree\Database\Query_Select::__construct
	 * @covers \Tree\Database\Query_Select::select
	 * @covers \Tree\Database\Query_Select::addColumn
	 * @covers \Tree\Database\Query_Select::from
	 * @covers \Tree\Database\Query_Select::addJoin
	 * @covers \Tree\Database\Query_Select::getSql
	 * @covers \Tree\Database\Query_Select::getJoinExpressions
	 */
	public function testMultipleComplexJoins()
	{

		$query = new Query_Select($this->db);
		$query->select('*');
		$query->from(array('sometable' => 'st'));

		$join = new Query_Join($this->db);
		$join->setTable('othertable', 'ot');
		$join->setType('LEFT');
		$join->on('st.ot_id = ot.ot_id');

		$query->addJoin($join);

		$join = new Query_Join($this->db);
		$join->setTable('thirdtable', 'tt');
		$join->setType('LEFT OUTER');
		$join->on('st.tt_id = tt.tt_id');
		$join->orOn("tt.tt_date > %s", '2010-01-01');

		$query->addJoin($join);

		$expected  = "SELECT *\n";
		$expected .= "FROM `sometable` `st`\n";
		$expected .= "LEFT JOIN `othertable` `ot`\n";
		$expected .= "ON st.ot_id = ot.ot_id\n";
		$expected .= "LEFT OUTER JOIN `thirdtable` `tt`\n";
		$expected .= "ON st.tt_id = tt.tt_id\n";
		$expected .= "OR tt.tt_date > '2010-01-01'\n";

		$actual = $query->getSql();

		$this->assertEquals($expected, $actual);
	}

}

